* Program   : BGEVENTS.PRG
* Author    : Robert Franklin Lamping
* Company   : FourDim
* Email     : robert@winning.com        (Lifetime Email address)
* Date      : May 8, 1997

* Clipper
#include "inkey.ch"

* Background events
#include "bgevents.ch"

#command	DEFAULT <p> TO <val> [,<pn> TO <valn> ] ;
		=> ;
		<p> = iif( <p> = NIL, <val>, <p> ) ;
		[;<pn> = iif( <pn> = NIL, <valn>, <pn> ) ]

#command	ADJUST <p> TO <val> [,<pn> TO <valn> ] ;
		=> ;
		<p> := iif( <val> = NIL, <p>, <val> ) ;
		[;<pn> := iif( <valn> = NIL, <pn> ,<valn> ) ]

#command	ADJUST <p> TO <val> IFNONIL <checkval>  ;
		=> ;
		<p> := iif( <checkval> = NIL, <p>, <val> )


* The following Globals Statics are postfixed with _BGE to see the difference
* between local variables and the global statics.
STATIC         aEvents_BGE := {}
STATIC       bIdleFunc_BGE := {|b| AT_OnIdle( b ) }
STATIC        lExecAll_BGE := .T.
STATIC     lInitActive_BGE := .T.
STATIC        lStopped_BGE := .F.
STATIC        lRunning_BGE := .F.
STATIC          lDEBUG_BGE := .T.  // .F.: events disabled when debugger linked in.
STATIC       nMaintick_BGE := 0
STATIC       nCurEvent_BGE := 0
STATIC     lKillSystem_BGE := .F.
Static bReentrantState_BGE := {||NIL}
STATIC          nARRAYLOCK := 0

Static InkeyEvBlock := {| b | Nil }

Static nLevel := 0

Static Function BGE_Init( aEvent )
   aEvent := Array( BGE_NLENGTH )
   AEVENT[ BGE_NHANDLE             ] := 0
   AEVENT[ BGE_BACTION             ] := {|| NIL }
   AEVENT[ BGE_BONDEACTIVATION     ] := {|| NIL }
   AEVENT[ BGE_BONACTIVATION       ] := {|| NIL }
   AEVENT[ BGE_LACTIVATED          ] := .F.        // PERFORMED ONCE AFTER ACTIVATION?
   AEVENT[ BGE_CNAME               ] := ""
   AEVENT[ BGE_LACTIVE             ] := .T.
   AEVENT[ BGE_NINTERVAL           ] := 0
   AEVENT[ BGE_BWHEN               ] := {|| .T. }
   AEVENT[ BGE_LKILLIT             ] := .F.
   AEVENT[ BGE_LKILLITAFTER        ] := .F.
   AEVENT[ BGE_NSECONDS            ] := 0  // NEEDED FOR INTERVAL
   AEVENT[ BGE_NTIMER              ] := 0
   AEVENT[ BGE_LINPROGRESS         ] := .F.
   AEVENT[ BGE_NPOSLOCK            ] := BGE_lockkey()
   AEVENT[ BGE_LEMPTY              ] := .F.
   AEVENT[ BGE_LSTOP               ] := .F.
   AEVENT[ BGE_LDIRECT             ] := .F.

   Return aEvent

INIT FUNCTION INIT_BGE
   * overruling default event-handler:  user Nanforum instead
   * Un-comment if needed and link in Nanforum lib
//   bIdleFunc_BGE := {|b| FT_OnIdle( b ) }
   Return .T.

EXIT FUNCTION EXIT_BGE
   * Stop inclusion of event in main-loop
   Eval( bIdleFunc_BGE )
   Return .T.

Function BGE_POS( xHandle )
   Local nPos := 0

   If Valtype( xHandle ) == "N"
      nPos := aScan( aEvents_BGE, {|el| el[ BGE_NHANDLE ] == xHandle} )
   Elseif Valtype( xHandle ) == "C"
      xHandle := Upper( Alltrim( xHandle ) )
      nPos := aScan( aEvents_BGE, {|el| el[ BGE_CNAME ] == xHandle} )
   Endif
   Return nPos

Function BGE_DEBUG( lDebug )
   If valtype( lDebug ) == "L"
      lDEBUG_BGE := lDebug
   Endif
   Return 0

Function BGE_DSeconds( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle ) IfNoNil xHandle
   IF nPos > 0
      Return aEvents_BGE[ nPos, BGE_NSECONDS ] 
   Endif
   Return 0

Function BGE_Seconds( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle ) IfNoNil xHandle
   IF nPos > 0
      Return aEvents_BGE[ nPos, BGE_NSECONDS ]
   Endif
   Return 0

Function BGE_TIMER( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle )     IfnoNil xHandle
   * Return value of nTimer
   IF nPos > 0
      Return aEvents_BGE[ nPos, BGE_NTIMER ]
   Endif
   Return Nil

Function BGE_INTERVAL( xHandle, nPos, nNewValue )
   Local nOldvalue := 0
   Adjust nPos to BGE_POS( xHandle )     IfnoNil xHandle

   * Return value of nTimer
   IF nPos > 0
      nOldValue := aEvents_BGE[ nPos, BGE_NINTERVAL ]
      If Valtype( nNewValue ) == "N"
         aEvents_BGE[ nPos, BGE_NINTERVAL  ] := nNewValue
      Endif
   Endif
   Return nOldValue

Function BGE_ResetTimer( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle )     IfnoNil xHandle
   IF nPos > 0
      aEvents_BGE[ nPos, BGE_NSECONDS ] := DSeconds()
      aEvents_BGE[ nPos, BGE_NTIMER   ] := DSeconds()
   Endif
   Return Nil

Function BGE_Start()
  If ! lRunning_BGE
     If Len( aEvents_BGE ) > 0

        If lDEBUG_BGE
           Eval( bIdleFunc_BGE )
           Eval( bIdleFunc_BGE, {|| BGE_EXEC() } )
        Endif

        lRunning_BGE := .T.
        lStopped_BGE := .F.

     Endif
  Endif
  Return .T.

Function BGE_Stop
   Local lOldvalue := lRunning_BGE
   Eval( bIdleFunc_BGE )
   lStopped_BGE := .T.
   lRunning_BGE := .F.
   Return lOldValue

Function BGE_AutoActive( lActive )
   * Set Initial Activness of the events
   Default lActive to .T.
   lInitActive_BGE := lActive
   Return Nil

Function BGE_MAINTICK( nInterval )
   If Valtype( nInterval ) == "N"
      nMainTick_BGE := nInterval
   Endif
   Return Nil

Function BGE_EXECALL( lLogic )
   If Valtype( lLogic ) == "L"
      lExecAll_BGE := lLogic
   Endif
   Return Nil

Function BGE_IDLEFUNC( bIdle  )
   Local xOldValue := bIdleFunc_BGE
   If Valtype( bIdle ) == "B"
      bIdleFunc_BGE := bIdle
   Endif
  Return xOldValue

Function BGE_bReSTATE( bBlock  )
   Local xOldValue := bReentrantState_BGE
   If Valtype( bBlock ) == "B"
      bReentrantState_BGE := bBlock
   Endif
  Return xOldValue


Function BGE_Stall( xHandle, nPos )
  Adjust nPos to BGE_POS( xHandle )     IfnoNil xHandle

  If aEvents_BGE[ nPos , BGE_LACTIVE ]
     Eval( aEvents_BGE[ nPos, BGE_BONDEACTIVATION ] )
  Endif
  aEvents_BGE[ nPos , BGE_LSTOP      ] := .T.
  aEvents_BGE[ nPos , BGE_LACTIVE    ] := .F.
  aEvents_BGE[ nPos , BGE_LACTIVATED ] := .T.

  Return .T.

Static Function BGE_Cloack( xHandle, nPos )  // nPos overrules xHandle
                                             // be sure nPos is ok
                                             // there is no range-check
  adjust nPos to BGE_POS( xHandle ) IFNONIL xHandle

  aEvents_BGE[ nPos , BGE_LACTIVE    ] := .F.
  Return .T.

Static Function BGE_DeCloack( xHandle, nPos )
  adjust nPos to BGE_POS( xHandle )  IfnoNil xHandle
  If ! aEvents_BGE[ nPos , BGE_LSTOP    ]
     aEvents_BGE[ nPos , BGE_LACTIVE    ] := .T.
  Endif
  Return .T.

Function BGE_Resume( xHandle, nPos )
  adjust nPos to BGE_POS( xHandle )  IfnoNil xHandle
  aEvents_BGE[ nPos , BGE_LSTOP    ]   := .F.
  aEvents_BGE[ nPos , BGE_LACTIVE    ] := .T.
  aEvents_BGE[ nPos , BGE_LACTIVATED ] := .F.
  Return .T.

Static Function BGE_OnAct( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle ) IfnoNil xHandle

   If ! aEvents_BGE[ nPos , BGE_LACTIVATED ]

      aEvents_BGE[ nPos , BGE_LACTIVATED ] := .T.

      Eval(aEvents_BGE[ nPos , BGE_BONACTIVATION ] )

      * Reset timers: interval-timer and start-timer
      aEvents_BGE[ nPos , BGE_NSECONDS ] := Dseconds()
      aEvents_BGE[ nPos , BGE_NTIMER   ] := aEvents_BGE[ nPos , BGE_NSECONDS ]
   Endif
   Return .T.

*-----------------------------------------------------------------

Function BGE_new( pcName, pbBlock, pnInterval, pbWhen, plOnce, plInActive, pbDeaction, pbOnActivation, lDirect )
   Local nHandle:= 0
   Local aEvent := {}
   Local nPos   := 0

   * Create new Event
   BGE_INIT( @aEvent )  // set initial values

   IF BGE_ARRAYLOCK()  // keep trying forever

      Do WHILE .T.
         nPos := aScan( aEvents_BGE, {|el| el[ BGE_LEMPTY ] } )
         If nPos == 0
            Exit
         Elseif BGE_POSLOCK( nPos, 5 )
            * Try to claim array element position
            Exit
         Endif
      Enddo

      If nPos = 0
         AAdd( aEvents_BGE, aEvent )
         nPos := Len ( aEvents_BGE )
      Else
         aEvents_BGE[ nPos ] := aEvent
      Endif
      * Array Element is still locked

      * Create new handle by looking up highest nHandle
      aEval( aEvents_BGE, {|el| nHandle := Max( nHandle, el[ BGE_NHANDLE ] ) } )
      nHandle++

      * Adjusting values
      Adjust aEvent[ BGE_BACTION ]          to pbBlock
      Adjust aEvent[ BGE_BONDEACTIVATION ]  to pbDeAction
      Adjust aEvent[ BGE_BONACTIVATION ]    to pbOnActivation

      If ! Empty( pcName )
         adjust aEvent[ BGE_CNAME ] to Upper( alltrim( pcName))
      Endif
      adjust aEvent[ BGE_BWHEN ]     to pbWhen
      adjust aEvent[ BGE_NINTERVAL ] to pnInterval
      adjust aEvent[ BGE_LDIRECT ]   to lDirect

      If ValType( plInActive ) == "L"
         aEvent[ BGE_LACTIVE ] := ! plInActive
      else
         Adjust aEvent[ BGE_LACTIVE ] to lInitActive_BGE
      Endif

      Adjust aEvent[ BGE_LKILLITAFTER ] to plOnce
      aEvent[ BGE_NSECONDS ] := Dseconds()
      BGE_Start()

      BGE_POSUNLOCK( nPos)
      BGE_ARRAYUNLOCK()
   ENDIF

   Return  ( aEvent[ BGE_NHANDLE ] )

*-----------------------------------------------------------------

Function BGE_FindByName( cName )  // obsolete
   Local nPos := BGE_POS( cName )
   if nPos > 0
      Return aEvents_BGE[ nPos ]
   endif
   Return Nil


Function BGE_Kill( xHandle, nPos)
   * Deactivate event first
   * Lock array .. try forever
   Local lDone := .F.

   Adjust nPos to BGE_POS( xHandle ) IfnoNil xHandle

   if BGE_PosLock( nPos )

      aEvents_BGE[ nPos, BGE_LEMPTY  ] := .T.  // may be reused by BGE_NEW
      aEvents_BGE[ nPos, BGE_LACTIVE ] := .T.
      aEvents_BGE[ nPos, BGE_NHANDLE ] := 0

      lDone := .T.
      BGE_PosUnLock( nPos )
   Endif
   Return lDone


Static Function BGE_PosLock( nPos, nWait )
   * Locking an array element is necessary, because of the event-handler
   * is reentrant and more procedure might want to approach the same element.

   * Try to lock array element forever
   Local nLock := BGE_lockkey()
   Local nSeconds := Dseconds()
   Local lForever
   Local lLocked := .F.
   Default nWait to 0

   lForever := (nWait == 0)

   If nPos > 0
      While .T.

         If aEvents_BGE[ nPos, BGE_NPOSLOCK ] != nLock
            If aEvents_BGE[ nPos, BGE_NPOSLOCK ] = 0
               aEvents_BGE[ nPos, BGE_NPOSLOCK ] := nLock
               if aEvents_BGE[ nPos, BGE_NPOSLOCK ] == nLock
                  lLocked := .T.
                  Exit
               Endif
            Endif
         Endif

         If ! lForEver
            If DSeconds() - nSeconds > nWait
               Exit
            Endif
         Endif

      End
   Endif
   Return lLocked

Static Function BGE_PosUnLock( nPos )
  * Unlock array element
  aEvents_BGE[ nPos, BGE_NPOSLOCK ] := 0
  Return .T.


Static Function BGE_ARRAYUNLOCK()
  * Unlock array element
  nARRAYLOCK := 0
  Return .T.

Static Function BGE_ARRAYLOCK()
   * Try to lock array forever
   Local nLock := BGE_lockkey()
      While nArrayLock != nLock
         If nArrayLock = 0
            nArrayLock := nLock
            if nArrayLock == nLock
               Exit
            Endif
         Endif
      End
      Return .T.
   Return .F.


Function BGE_RELEVEL()
  /*
  * Return Reentrance Level
  */
  Return nLevel


Function BGE_Exec()
  Local col     := col()
  Local row     := row()
  Local nCurSor := setcursor()
  Local lAllow  := .F.
  Static nSeconds
  If Valtype( nSeconds ) != "N"
    nSeconds := DSeconds()
  Endif
  
  ++nLevel
  If Dseconds() - nSeconds > nMainTick_BGE
    nSeconds := Dseconds()
    lAllow := .T.
  Endif
  If lAllow

     Eval( BGE_BRESTATE(), nLevel )

     devpos(row, col)

     If lExecAll_BGE

        aEval( aEvents_BGE, {|el, pos|  BGE_BEXEC( pos )  } )

     Else
        If nCurEvent_BGE = 0
          nCurEvent_BGE := 1
        Endif

        BGE_BEXEC( nCurEvent_BGE )
        If ++nCurEvent_BGE > Len( aEvents_BGE )
           nCurEvent_BGE := 1
        Endif
     Endif


     devpos(row, col)
     setcursor( nCursor )

  Endif
  nLevel--

  Return ( 0 )

Function BGE_bexec( nPos )
  Local nSec := 0
  Local aEvent := aEvents_BGE[ nPos ]

  aEvent[ BGE_LINPROGRESS ]  := .T.

  if aEvent[ BGE_LKILLIT ]
    BGE_Kill( , nPos )

  elseif aEvent[ BGE_LACTIVE ]

     If Eval( aEvent[ BGE_BWHEN ] )
        if aEvent[ BGE_NINTERVAL ] > 0

          nSec := Dseconds()

          If ( nSec - aEvent[ BGE_NSECONDS ] > aEvent[ BGE_NINTERVAL ] ) .OR. ;
             ( aEvent[ BGE_LDIRECT ] .and. ! aEvent[ BGE_LACTIVATED ] )

             aEvent[ BGE_NSECONDS ] := nSec

             BGE_Cloack( , nPos )
             BGE_OnAct( , nPos )
             Eval( aEvent[ BGE_BACTION ]  )
             BGE_DeCloack( , nPos )
          Endif

        Else
             BGE_Cloack( , nPos )
             BGE_OnAct( , nPos )
             Eval( aEvent[ BGE_BACTION ]  )
             BGE_DeCloack( , nPos )

        Endif
     Endif
  Endif

  if nPos <= Len( aEvents_BGE )
    aEvent[ BGE_LINPROGRESS ]  := .F.
  Endif

  Return .t.

Function BGE_RELEASE( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle ) IfnoNil xHandle
   IF nPos > 0
     aEvents_BGE[ nPos, BGE_LKILLIT ] := .T.
   Endif
   Return nPos > 0

Function BGE_ACT( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle ) IfnoNil xHandle
   IF nPos > 0
      BGE_Resume( , nPos )
   Endif
   Return nPos > 0

Function BGE_DEACT( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle )     IfnoNil xHandle
   IF nPos > 0
      BGE_Stall( , nPos )
   Endif
   Return nPos > 0

Function BGE_EVENT( xHandle, nPos )
   Adjust nPos to BGE_POS( xHandle )     IfnoNil xHandle
   Return aEvents_BGE[ nPos ]

Function BGE_CLEAR()  // not active et
   lKILLSYSTEM_BGE := .T.
   Return .T.

Function AT_onIdle( bBlock )
  If Empty( bBlock )
     EvhClear( bBlock )
  Else
     EvhSet( bBlock )
  ENdif
  Return nil

Static Function BGE_lockkey()
  Static nlockkey := 0
  nlockkey++
  If nlockkey > 65000
     nlockkey := 1
  Endif
  Return nLockkey

Function DSECONDS()
   Return Int( Date()-ctod("01/01/1000")) * 86400 +  seconds()

Function Inkey_onIdle( bBlock )
  If Empty( bBlock )
     InkeyEvBlock := {|| Nil }
  Else
     InkeyEvBlock := bBlock
  ENdif
  Return nil

Function InkeyEvents()  
  * Call this function in your function MyInkey() or whereever you like!
  Return  InkeyEvBlock 

