/*
 Events.h
 Low-level interface met CA-Clipper's event systeem.
 Met dank aan Ian Day, maker van Mr. Debug.
*/

#ifndef EVENTS_DEFINED

typedef struct {
   short int row;
   short int col;
} POINT;

typedef POINT far * POINTP;


typedef struct {
	USHORT etype;
	USHORT eflags;

   union {
      struct {
			USHORT button;
			POINT pt;
		};                // Muis invoer

      struct {
			USHORT key;
			USHORT vkey;
			USHORT ksflags;
		};                // Toetsenbord invoer
	};
} LOWEVENT;

typedef LOWEVENT far * LOWEVENTP;


typedef struct {
	USHORT qEntries;
	USHORT qSize;
	BYTEP queue;
} LOWQUEUESPEC;

typedef LOWQUEUESPEC far * LOWQUEUESPECP;


/* low event masks (eveSetMask) */
#define INKEY_MOVE        0x01    //   Mouse movements
#define INKEY_LDOWN       0x02    //   Mouse left button down
#define INKEY_LUP         0x04    //   Mouse left button up
#define INKEY_RDOWN       0x08    //   Mouse right button down
#define INKEY_RUP         0x10    //   Mouse right button up
#define INKEY_MDOWN       0x20    //   Mouse middle button down
#define INKEY_MUP         0x40    //   Mouse middle button up

#define INKEY_KEYBOARD    0x80    //   Keyboard events

#define INKEY_ALL         0x8F

#define INKEY_L               ( INKEY_LUP   | INKEY_LDOWN )
#define INKEY_R               ( INKEY_RUP   | INKEY_RDOWN )
#define INKEY_M               ( INKEY_MUP   | INKEY_MDOWN )
#define INKEY_BUTTON_PRESS    ( INKEY_LDOWN | INKEY_RDOWN | INKEY_MDOWN )
#define INKEY_BUTTON_RELEASE  ( INKEY_LUP   | INKEY_RUP   | INKEY_MUP )
#define INKEY_ON              TRUE     /* set to receive */
#define INKEY_OFF             FALSE    /* set to ignore */

/* low break events */
#define ET_CTRL_BREAK              -1
#define ET_TERM_BREAK              -2
#define ET_SCRL_BREAK              -3
#define ET_DEBUG_BREAK             -4
#define ET_USER_BREAK0             -5
#define ET_USER_BREAK1             -6
#define ET_USER_BREAK2             -7
#define ET_USER_BREAK3             -8
#define ET_USER_BREAK4             -9
#define ET_USER_BREAK5            -10
#define BREAK_COUNT                10

/* low hardware events */
#define ET_NULLEVENT                0
#define ET_MOUSEDOWN                1
#define ET_MOUSEDBLCLICK            2
#define ET_MOUSEUP                  3
#define ET_MOUSEMOVE                4
#define ET_KEYSTROKE                5

/* low event "ekey" field masks */
#define EKM_CHARCODE                0x00FF
#define EKM_SCANCODE                0xFF00

/* low event "eflags" field masks */
#define EFM_RIGHTSHIFT              0x0001
#define EFM_LEFTSHIFT               0x0002
#define EFM_CTRLSHIFT               0x0004
#define EFM_ALTSHIFT                0x0008
#define EFM_SCROLLOCK               0x0010
#define EFM_NUMLOCK                 0x0020
#define EFM_CAPSLOCK                0x0040
#define EFM_INSERT                  0x0080
#define EFM_LEFTCTRL                0x0100
#define EFM_LEFTALT                 0x0200
#define EFM_RIGHTCTRL               0x0400
#define EFM_RIGHTALT                0x0800
#define EFM_SCROLL                  0x1000
#define EFM_NUM                     0x2000
#define EFM_CAPS                    0x4000
#define EFM_SYSRQ                   0x8000

/* _evLow subfunctions */
#define EVE_SetMask						1
#define EVE_SetQueue                2
#define EVE_PostEvent					3
#define EVE_Strobe						4
#define EVE_RegisterStrobe				5
#define EVE_RegisterQueueMonitor		6
#define EVE_RegisterBreakHandler		7
#define EVE_RegisterCriticalHandler 8
#define EVE_ReadQueue					9
#define EVE_ReadQueueRaw				10
#define EVE_ClearQueue					11
#define EVE_SetMinEvents				12

/* function call macros */
#define eveSetMask(m,b)                   _evLow (EVE_SetMask, (USHORT)m, (int)b)
#define eveSetQueue(qP)                   _evLow (EVE_SetQueue, (LOWQUEUESPECP)qP)
#define evePostEvent(eP)                  _evLow (EVE_PostEvent, (LOWEVENTP)eP)
#define eveStrobe()                       _evLow (EVE_Strobe)
#define eveRegisterStrobe(fP,b)           _evLow (EVE_RegisterStrobe, (FUNCP)fP, (int)b)
#define eveRegisterQueueMonitor(fP,b)     _evLow (EVE_RegisterQueueMonitor, (FUNCP)fP, (int)b)
#define eveRegisterBreakHandler(fP,b)     _evLow (EVE_RegisterBreakHandler, (FUNCP)fP, (int)b)
#define eveRegisterCriticalHandler(fP,b)  _evLow (EVE_RegisterCriticalHandler, (FUNCP)fP, (int)b)
#define eveReadQueue(eP)                  _evLow (EVE_ReadQueue, (LOWEVENTP)eP)
#define eveReadQueueRaw(eP)               _evLow (EVE_ReadQueueRaw, (LOWEVENTP)eP)
#define eveClearQueue()                   _evLow (EVE_ClearQueue)
#define eveSetMinEvents(n)                _evLow (EVE_SetMinEvents, (USHORT)n)

long _evLow ( USHORT fNum, ... );

#define EVENTS_DEFINED

#endif
